"""Internal utilities"""

import os
import re


def uri_is_tweet(uri: str) -> bool:
    """Return whether a URI points to a single tweet"""
    return bool(re.match("https\:\/\/twitter\.com\/.*\/status\/[0-9]+", uri))


def tweeturi2tweetid(uri: str) -> str:
    """Given a tweet URI, return just the tweet ID it contains"""
    m = re.match("https\:\/\/twitter\.com\/.*\/status\/(?P<tweetid>[0-9]+)", uri)
    return m["tweetid"]


def guess_mime_type(filename: str) -> str:
    """Given a filename, guess its MIME type. Works with URIs as well."""
    # This is just a random list I found, lol
    mime_types = dict(
        # images
        png="image/png",
        jpe="image/jpeg",
        jpeg="image/jpeg",
        jpg="image/jpeg",
        gif="image/gif",
        bmp="image/bmp",
        ico="image/vnd.microsoft.icon",
        tiff="image/tiff",
        tif="image/tiff",
        svg="image/svg+xml",
        svgz="image/svg+xml",
        # video
        mp4="video/mp4",
        qt="video/quicktime",
        mov="video/quicktime",
        # audio
        mp3="audio/mpeg",
        ogg="audio/ogg",
    )

    extension = os.path.splitext(filename)[1][1:].lower()

    # Some Twitter media URIs have a query string at the end, like
    # <https://video.twimg.com/ext_tw_video/1221557894694559750/pu/vid/320x568/Vnzq6WB09j9CMLOc.mp4?tag=10>

    extension = re.sub("\?.*", "", extension)

    return mime_types[extension]



"""Functions that relate to a Hugo blog"""

import datetime
import json
import os
import pathlib
import re
import subprocess
import textwrap
import typing


class HugoSite:
    def __init__(self, base: str):
        self.base = pathlib.Path(base)
        self.content = os.path.join(self.base, "content")
        self.content_twarchive = os.path.join(self.content, "twarchive")
        self.data = os.path.join(self.base, "data")
        self.data_twarchive = os.path.join(self.data, "twarchive")
        self.twitter_archives = os.path.join(self.base, "twitter-archives")


def find_inline_tweets(site: HugoSite) -> typing.List[str]:
    """Find tweets inlined with 'twarchiveTweet' or 'twarchiveThread' shortcodes in site content"""

    result = []

    rgcmd = [
        "rg",
        "--no-filename",
        "--no-line-number",
        r"\{\{. twarchiveTweet .[0-9]+(-intentionallyinvalid)?",
        site.content,
    ]
    rg = subprocess.run(rgcmd, capture_output=True)
    outlines = rg.stdout.decode().strip().split("\n")
    for line in outlines:
        if not line:
            continue
        m = re.search(
            "{{. twarchiveTweet .(?P<tweetid>[0-9]+(:?-intentionallyinvalid)?)", line
        )
        result += [m["tweetid"]]

    rgcmd = [
        "rg",
        "--no-filename",
        "--no-line-number",
        r"\{\{. twarchiveThread .[0-9]+(-intentionallyinvalid)?",
        site.content,
    ]
    rg = subprocess.run(rgcmd, capture_output=True)
    outlines = rg.stdout.decode().strip().split("\n")
    for line in outlines:
        if not line:
            continue
        m = re.search(
            "{{. twarchiveThread .(?P<tweetid>[0-9]+(:?-intentionallyinvalid)?)", line
        )
        result += [m["tweetid"]]

    return set(result)


def get_downloaded_tweets(site: HugoSite) -> typing.List[str]:
    """Return a list of tweet IDs that have already been downloaded"""
    return [t.strip(".json") for t in os.listdir(site.data_twarchive)]


def showinlines(site: HugoSite):
    inlines = find_inline_tweets(site)
    print("Note: these tweets may QT other tweets, which are not listed here")
    for line in inlines:
        print(f"- {line}")


def data2md(site: HugoSite):
    """For tweets that have been downloaded to the data directory, make a page for them in the content"""
    os.makedirs(site.content_twarchive, exist_ok=True)
    for tweet_json in os.listdir(site.data_twarchive):
        tweet_json_path = os.path.join(site.data_twarchive, tweet_json)
        with open(tweet_json_path) as tjfp:
            tweet = json.load(tjfp)
        tweet_date = datetime.datetime.strptime(tweet["date"], "%Y-%m-%dT%H:%M:%S%z")
        tweetid = tweet_json.strip(".json")
        tweet_md_path = os.path.join(site.content_twarchive, f"{tweetid}.md")
        mdcontents = textwrap.dedent(
            f"""\
            ---
            tweetid: "{tweetid}"
            date: {tweet_date}
            ---
            """
        )
        with open(tweet_md_path, "w") as tmdfp:
            tmdfp.write(mdcontents)



"""Functionality that involves talking to the Twitter API"""

import os

import tweepy

from twarchive import hugo
from twarchive import logger
from twarchive.inflatedtweet.inflatedtweet import InflatedTweet
from twarchive.inflatedtweet.from_tweepy import inflated_tweet_from_tweepy


def authenticate(consumer_key: str, consumer_secret: str) -> tweepy.API:
    auth = tweepy.OAuthHandler(consumer_key, consumer_secret)
    api = tweepy.API(auth)
    return api


def get_status_expanded(api: tweepy.API, tweetid: str) -> tweepy.models.Status:
    """Get a status"""
    tweet = api.get_status(
        tweetid,
        include_ext_alt_text=True,
        tweet_mode="extended",
    )
    return tweet


def tweetid2json(api: tweepy.API, tweetid: str, filename: str):
    tweet = get_status_expanded(api, tweetid)
    infltweet = InflatedTweet.from_tweet(tweet)
    infltweet.jdump(filepath=filename)


def tweet2data(
    site: hugo.HugoSite,
    api: tweepy.API,
    tweetid: str = "",
    tweet: tweepy.models.Status = None,
    force=False,
    rlevel=0,
    max_rlevel=20,
):
    """Save a tweet to site data

    Save its QTs as well.

    Arguments:
        site:               A hugo.HugoSite object
        api:                A tweepy.API that has already authenticated
        tweetid:            The ID of a tweet to save
        tweet:              An already retrieved Status object from tweepy
        force:              Redownload a tweet that already exists in site data.
        rlevel:             The recursion level of this function call.
                            This function calls itself for each QT, incrementing this value each time.
                            At some level we will stop, in case we are in some pathological case.
        max_rlevel:         The maximum number of QT or parent tweets to retrieve.
    """

    if (tweetid and tweet) or (not tweetid and not tweet):
        raise Exception("Must provide exactly one of tweet or tweetid")

    if not tweetid:
        tweetid = tweet.id_str

    filename = os.path.join(site.data_twarchive, f"{tweetid}.json")

    if os.path.exists(filename) and not force:
        logger.info(
            f"Tweet {tweetid} already exists in site data and force=False, not downloading"
        )
        return

    if rlevel > max_rlevel:
        logger.warning(
            f"WARNING! Recursion level of {rlevel}, refusing to download tweet {tweetid}"
        )
        return

    if not tweet:
        logger.info(f"Downloading tweet {tweetid}")
        tweet = get_status_expanded(api, tweetid)

    infltweet = inflated_tweet_from_tweepy(tweet)
    infltweet.jdump(filepath=filename)
    rlevel += 1

    related_tweets = []
    related_tweets.extend(infltweet.qts)
    if infltweet.thread_parent_id:
        related_tweets.append(infltweet.thread_parent_id)
    try:
        related_tweets.append(tweet.retweeted_status.id_str)
    except AttributeError:
        pass

    for reltweet in related_tweets:
        tweet2data_continue_on_error(
            site,
            api,
            tweetid=reltweet,
            force=force,
            rlevel=rlevel,
            max_rlevel=max_rlevel,
        )


def tweet2data_continue_on_error(
    site: hugo.HugoSite,
    api: tweepy.API,
    tweetid: str = "",
    tweet: tweepy.models.Status = None,
    force=False,
    rlevel=0,
    max_rlevel=20,
):
    """Call tweet2data() with the parameters, and handle exceptions from missing/removed tweets

    Tweepy will raise exceptions if a tweet has been deleted or an account has gone private.
    Catch and log these exceptions, then continue.
    """
    try:
        tweet2data(
            site,
            api,
            tweetid=tweetid,
            tweet=tweet,
            force=force,
            rlevel=rlevel,
            max_rlevel=max_rlevel,
        )
    except tweepy.errors.NotFound:
        # This appears to mean the tweet (or account) were intentionally deleted
        logger.warning(f"Could not find tweet with ID {tweetid}")
    except tweepy.errors.Forbidden as exc:
        # This appears to happen when the account was suspended
        exc_message = str(exc).replace("\n", " ")
        logger.warning(
            f"Not permitted to access tweet with ID {tweetid}: {exc_message}"
        )


def usertweets2data(
    site: hugo.HugoSite,
    api: tweepy.API,
    screen_name: str,
    max_rlevel: int,
    force: bool = False,
    retrieve_all: bool = False,
):
    """Save all of a user's tweets to data

    Arguments
    api:                A tweepy API object, already logged in
    screen_name:        The username to retrieve
    max_rlevel:         Max recursion level for QTs/replies
    force:              Download data again even if we already have it
    retrieve_all:       When True, go all the way to the beginning of the user's timeline.
                        When False (default), stop when encountering a tweet we've gotten before.
                        If force is True, this has no extra effect.
                        If force is False, the program will not redownload QTs or media
                        for tweets it already has, but will check all tweets the Twitter API lets us retrieve.
    """

    # Twitter API limit
    max_tweets_per_call = 200

    downloaded_tweets = hugo.get_downloaded_tweets(site)
    user_tweets = []
    oldest = None
    while True:
        logger.info(f"Retrieving tweets for @{screen_name} older than {oldest}")
        new_tweets = api.user_timeline(
            screen_name=screen_name,
            count=max_tweets_per_call,
            tweet_mode="extended",
            max_id=oldest,
        )
        if not new_tweets:
            break

        if force:
            oldest = str(new_tweets[-1].id - 1)
            continue

        unseen_tweets = [t for t in new_tweets if t.id_str not in downloaded_tweets]
        user_tweets.extend(unseen_tweets)
        if not retrieve_all and len(unseen_tweets) < len(new_tweets):
            break
        oldest = str(new_tweets[-1].id - 1)

    logger.info(
        f"Finished retrieving tweets for @{screen_name}, got {len(user_tweets)} tweets (force={force}, retrieve_all={retrieve_all})"
    )

    for tweet in user_tweets:
        tweet2data(site, api, tweet=tweet, force=force, max_rlevel=max_rlevel)



import argparse
import os
import pdb
import sys
import traceback
import typing

import tweepy

from twarchive import hugo
from twarchive import logger
from twarchive import twitterapi
from twarchive import twitterarchive
from twarchive import version
from twarchive.inflatedtweet.inflatedtweet import InflatedTweet


def idb_excepthook(type, value, tb):
    """Call an interactive debugger in post-mortem mode

    If you do "sys.excepthook = idb_excepthook", then an interactive debugger
    will be spawned at an unhandled exception
    """
    if hasattr(sys, "ps1") or not sys.stderr.isatty():
        sys.__excepthook__(type, value, tb)
    else:
        traceback.print_exception(type, value, tb)
        print
        pdb.pm()


def inline2data(
    site: hugo.HugoSite, api: tweepy.API, max_rlevel: int, force: bool = False
):
    """Save tweets to data that have been inlined

    Does not redownload items already saved.
    """
    for tweetid in hugo.find_inline_tweets(site):
        if tweetid.endswith("-intentionallyinvalid"):
            logger.info(f"Skipping intentionally invalid tweet id {tweetid}")
        else:
            twitterapi.tweet2data(
                site, api, tweetid=tweetid, force=force, max_rlevel=max_rlevel
            )


def list_directory_tweetid_filename(directory: str) -> typing.List[str]:
    """List a directory of files with tweetid filenames and return them sorted.

    When sorting, zero pad all tweet IDs.

    Filenames that do not start with numbers are ignored.
    """

    def sortkey(s: str):
        return int(s.split(".")[0])

    children = os.listdir(directory)
    child_numbers = [c for c in children if c[0].isdigit()]
    child_numbers.sort(key=sortkey)
    return child_numbers


def parseargs():
    """Parse program arguments"""
    parser = argparse.ArgumentParser(description="Manage tweets")
    subparsers = parser.add_subparsers(dest="action", required=True)
    parser.add_argument(
        "--debug",
        "-d",
        action="store_true",
        help="Launch a debugger on unhandled exception",
    )

    ## Options related to the Twitter API
    twitter_opts = argparse.ArgumentParser(add_help=False)
    # Defaults are official keys for iOS app
    # https://gist.github.com/shobotch/5160017
    twitter_opts.add_argument(
        "--consumer-key", default="IQKbtAYlXLripLGPWd0HUA", help="Twitter consumer key"
    )
    twitter_opts.add_argument(
        "--consumer-secret",
        default="GgDYlkSvaPxGxC4X8liwpUoqKwwr3lCADbz8A7ADU",
        help="Twitter consumer secret",
    )
    twitter_opts.add_argument(
        "--max-recurse",
        type=int,
        default=20,
        help="How many parent/QT tweets should we find",
    )
    twitter_opts.add_argument(
        "--force",
        action="store_true",
        help="Redownload data that has already been downloaded",
    )

    ## Options related to a Hugo site
    hugo_opts = argparse.ArgumentParser(add_help=False)
    hugo_opts.add_argument(
        "--hugo-site-base",
        default=".",
        help="Path to a Hugo site. Defaults to the $PWD.",
    )

    ## Subcommand: version
    sub_version = subparsers.add_parser("version", help="Show program version")
    sub_version.add_argument(
        "--quiet",
        action="store_true",
        help="Show just the version (used in development)",
    )

    ## Subcommand: ls
    sub_ls = subparsers.add_parser(
        "ls",
        parents=[hugo_opts],
        help="List tweet files in a Hugo repository, sorted by tweet ID and accounting for tweet IDs of different lengths",
    )
    sub_ls.add_argument(
        "directory",
        choices=["data", "content"],
        help="Choose the directory to list - either data/twarchive or content/twarchive",
    )

    ## Subcommand: internals
    sub_internals = subparsers.add_parser(
        "internals", help="Internal commands used in development"
    )
    sub_internals_subparsers = sub_internals.add_subparsers(
        dest="internalsaction", required=True
    )

    ## Subcommand: internals: Subcommand: examine
    sub_internals_sub_examine = sub_internals_subparsers.add_parser(
        "examine",
        parents=[twitter_opts],
        help="Get the response from Twitter and launch the debugger",
    )
    sub_internals_sub_examine.add_argument("tweetid", help="ID of the tweet to examine")

    ## Subcommand: internals: Subcommand: json-load-dump-tweets
    # Useful for testing our en/de-coders
    # Also nice to ensure that all tweets were saved with indent=2 and sort_keys=True,
    # which is useful when saving the resulting JSON to git for diffing etc
    sub_internals_sub_json_load_dump_tweets = sub_internals_subparsers.add_parser(
        "json-load-dump-tweets",
        parents=[hugo_opts],
        help="Use our custom JSON en/de-coders to parse JSON of every tweet saved to data/twarchive/*.json, and save it back out.",
    )

    ## Subcommand: tweet2json
    sub_tweet2json = subparsers.add_parser(
        "tweet2json",
        parents=[twitter_opts, hugo_opts],
        help="Download JSON for a tweet, including image data",
    )
    sub_tweet2json.add_argument("tweetid", help="ID of a tweet to download")
    sub_tweet2json.add_argument("filename", help="Path to save JSON to")

    ## Subcommand: tweet2data
    sub_tweet2data = subparsers.add_parser(
        "tweet2data",
        parents=[twitter_opts, hugo_opts],
        help="Download JSON for a tweet, and store it under './data/twarchive/$tweetId.json', including any QTs or parents",
    )
    sub_tweet2data.add_argument("tweetid", help="ID of a tweet to download")

    ## Subcommand: showinlines
    sub_showinlines = subparsers.add_parser(
        "showinlines",
        parents=[hugo_opts],
        help="Show tweets inlined with the 'twarchiveTweet' shortcode",
    )

    ## Subcommand: inline2data
    sub_inline2data = subparsers.add_parser(
        "inline2data",
        parents=[twitter_opts, hugo_opts],
        help="Download JSON for all tweets that are inlined via the 'twarchiveTweet' shortcode.",
    )

    ## Subcommand: data2md
    sub_data2md = subparsers.add_parser(
        "data2md",
        parents=[hugo_opts],
        help="Create a page under content/twarchive/ for every downloaded tweet",
    )

    ## Subcommand: user2data
    sub_user2data = subparsers.add_parser(
        "user2data",
        parents=[twitter_opts, hugo_opts],
        help="Retrieve a user's full timeline from Twitter and store in site data",
    )
    sub_user2data.add_argument("username", help="Twitter username")
    sub_user2data.add_argument(
        "--retrieve-all",
        action="store_true",
        help="Keep retrieving tweets from this user even if we retrieve a tweet we already have. Has no effect if --force is True, but if --force is False, it will download older tweets without redownloading media/QTs from newer ones it already has.",
    )

    ## Subcommand: archive2data
    sub_archive2data = subparsers.add_parser(
        "archive2data",
        parents=[twitter_opts, hugo_opts],
        help="Parse a Twitter archive extracted to twitter-archives/<name> and save the result to the Hugo data dir",
    )
    sub_archive2data.add_argument(
        "--archive",
        help="Name of the archive to parse; if not present, find archives automatically under ./twitter-archives",
    )
    sub_archive2data.add_argument(
        "--no-api",
        action="store_true",
        help="Don't query the API for missing RTs, QTs, or thread parents",
    )

    ## Subcommand: show-inline-tweets
    sub_show_inline_tweets = subparsers.add_parser(
        "show-inline-tweets",
        parents=[hugo_opts],
        help="Show a list of all tweets included by twarchive Hugo partials",
    )

    ## Done
    parsed = parser.parse_args()
    return parser, parsed


def main():
    """Main program"""
    parser, parsed = parseargs()
    if parsed.debug:
        sys.excepthook = idb_excepthook

    if parsed.action == "version":
        if parsed.quiet:
            print(version.__version__)
        else:
            print(f"{parser.prog} version {version.__version__}")
        return 0

    elif parsed.action == "ls":
        site = hugo.HugoSite(parsed.hugo_site_base)
        if parsed.directory == "data":
            dir = site.data_twarchive
        elif parsed.directory == "content":
            dir = site.content_twarchive
        else:
            raise Exception(f"Unknown directory {parsed.directory}")
        children = [os.path.join(dir, c) for c in list_directory_tweetid_filename(dir)]
        print("\n".join(children))

    elif parsed.action == "internals":
        if parsed.internalsaction == "examine":
            api = twitterapi.authenticate(parsed.consumer_key, parsed.consumer_secret)
            tweet = twitterapi.get_status_expanded(api, parsed.tweetid)
            pdb.set_trace()
        elif parsed.internalsaction == "json-load-dump-tweets":
            site = hugo.HugoSite(parsed.hugo_site_base)
            for tweetjson in os.listdir(site.data_twarchive):
                tweetjson_path = os.path.join(site.data_twarchive, tweetjson)
                with open(tweetjson_path) as tjfp:
                    infltweet = InflatedTweet.jload(tjfp)
                with open(tweetjson_path, "w") as tjfp:
                    infltweet.jdump(tjfp)

    elif parsed.action == "tweet2json":
        api = twitterapi.authenticate(parsed.consumer_key, parsed.consumer_secret)
        twitterapi.tweetid2json(api, parsed.tweetid, parsed.filename)

    elif parsed.action == "tweet2data":
        site = hugo.HugoSite(parsed.hugo_site_base)
        api = twitterapi.authenticate(parsed.consumer_key, parsed.consumer_secret)
        twitterapi.tweet2data(
            site,
            api,
            tweetid=parsed.tweetid,
            force=parsed.force,
            max_rlevel=parsed.max_recurse,
        )
        hugo.data2md(site)

    elif parsed.action == "showinlines":
        site = hugo.HugoSite(parsed.hugo_site_base)
        hugo.showinlines(site)

    elif parsed.action == "inline2data":
        site = hugo.HugoSite(parsed.hugo_site_base)
        api = twitterapi.authenticate(parsed.consumer_key, parsed.consumer_secret)
        twitterapi.inline2data(
            site,
            api,
            force=parsed.force,
            max_rlevel=parsed.max_recurse,
        )
        hugo.data2md(site)

    elif parsed.action == "data2md":
        site = hugo.HugoSite(parsed.hugo_site_base)
        hugo.data2md(site)

    elif parsed.action == "user2data":
        site = hugo.HugoSite(parsed.hugo_site_base)
        api = twitterapi.authenticate(parsed.consumer_key, parsed.consumer_secret)
        twitterapi.usertweets2data(
            site,
            api,
            parsed.username,
            max_rlevel=parsed.max_recurse,
            force=parsed.force,
            retrieve_all=parsed.retrieve_all,
        )
        hugo.data2md(site)

    elif parsed.action == "archive2data":
        site = hugo.HugoSite(parsed.hugo_site_base)

        if parsed.archive:
            archives = [
                twitterarchive.TwitterArchive.frompath(
                    os.path.join(site.twitter_archives, parsed.archive)
                )
            ]
        else:
            archives = twitterarchive.find_archives(site)

        if parsed.no_api:
            api = None
        else:
            api = twitterapi.authenticate(parsed.consumer_key, parsed.consumer_secret)

        for archive in archives:
            twitterarchive.archive2data(
                site,
                archive,
                api=api,
                max_recurse=parsed.max_recurse,
                api_force_download=parsed.force,
            )

    elif parsed.action == "show-inline-tweets":
        site = hugo.HugoSite(parsed.hugo_site_base)
        for inline in hugo.find_inline_tweets(site):
            print(inline)

    else:
        raise Exception(f"Unknown action: {parsed.action}")



"""Functionality related to a downloaded Twitter archive

Twitter allows account owners to download all their own tweets,
as well as a bunch of other data,
in an archive file.
We can use any archive files as a definitive copy of a tweet.

Note that a tweet found in an archive should _never_ be redownloaded,
at least as long as Twitter does not permit edits.
"""

import datetime
from functools import cache
import functools
import json
import os
import re
import typing

import tweepy

from twarchive import hugo, logger, twitterapi
from twarchive.inflatedtweet.from_twitter_archive import (
    inflated_tweet_from_twitter_archive,
    twitter_archive_tweet_is_low_fidelity_retweet,
)
from twarchive.inflatedtweet.inflatedtweet import InflatedTweet


class TwitterArchive(typing.NamedTuple):
    """An on-disk, extracted Twitter archive

    Note: NamedTuple objects cannot be annotated as @functools.cached_property
    Instead, they can use stacked @property, @functools.cache decorators.
    <https://docs.python.org/dev/library/functools.html#functools.cached_property>
    """

    path: str
    manifestjs: str
    profilejs: str
    tweetjs: str
    accountjs: str
    profilemedia: str
    tweetmedia: str

    def anymissing(self) -> typing.List[str]:
        """Checks for existence of required files"""
        required = [
            self.manifestjs,
            self.profilejs,
            self.tweetjs,
            self.accountjs,
            self.profilemedia,
            self.tweetmedia,
        ]
        missing = [i for i in required if not os.path.exists(i)]
        return missing

    @property
    @functools.cache
    def manifest(self) -> typing.Dict:
        parsed_manifest = parse_twitter_manifest_js(self.manifestjs)
        return parsed_manifest

    @property
    @functools.cache
    def generation_date(self) -> datetime.datetime:
        gendate_str = self.manifest["archiveInfo"]["generationDate"]
        # eg: "generationDate" : "2022-05-06T17:27:17.889Z"
        # gendate_dt = datetime.datetime.fromisoformat(gendate_str)
        gendate_dt = datetime.datetime.strptime(gendate_str, r"%Y-%m-%dT%H:%M:%S.%fz")
        return gendate_dt

    @classmethod
    def frompath(cls, p: str) -> "TwitterArchive":
        data = os.path.join(p, "data")
        archive = cls(
            p,
            os.path.join(data, "manifest.js"),
            os.path.join(data, "profile.js"),
            os.path.join(data, "tweet.js"),
            os.path.join(data, "account.js"),
            os.path.join(data, "profile_media"),
            os.path.join(data, "tweet_media"),
        )
        return archive


def parse_twitter_manifest_js(manifest: str) -> typing.Dict:
    """The fucking Twitter archive MANIFEST is in a DIFFERENT fucking deranged format for no good reason"""
    with open(manifest) as mfp:
        contents = mfp.read()
    unfucked = re.sub(r"^window\.__THAR_CONFIG = ", "", contents)
    parsed = json.loads(unfucked)
    return parsed


def parse_twitter_window_YTD_bullshit(
    archivefile: str,
) -> typing.Union[typing.Dict, typing.List]:
    """The fucking Twitter archive is in a deranged format for no good reason."""
    with open(archivefile) as afp:
        contents = afp.read()
    unfucked = re.sub(
        r"^window\.YTD\.[a-zA-Z0-9_]+\.part[a-zA-Z0-9_]+ = ", "", contents
    )
    parsed = json.loads(unfucked)
    return parsed


def find_archives(site: hugo.HugoSite) -> typing.List[str]:
    """Find archives saved locally.

    Assumes archives are saved to ./twitter-archives in the root of a Hugo blog repo.
    (No better place for them, really;
    they include private data like DMs which should not be exposed
    and therefore they cannot go into static/,
    and Hugo's ./data only supports json/toml/etc,
    which will not work with the images saved as separate files in the Twitter archives.)
    """

    archives = []
    for archive in os.listdir(site.twitter_archives):
        archive = TwitterArchive.frompath(os.path.join(site.twitter_archives, archive))
        missing = archive.anymissing()
        if missing:
            logger.debug(
                f"Skipping archive {archive} as it is not a complete archive, missing: {missing}"
            )
            continue
        archives.append(archive)
    return archives


def archive2data(
    site: hugo.HugoSite,
    archive: TwitterArchive,
    api: typing.Optional[tweepy.API] = None,
    max_recurse=1,
    api_force_download=False,
):
    """Parse all the tweets in an archive and return a list of InflatedTweet objects"""

    os.makedirs(site.data_twarchive, exist_ok=True)

    parsed_account = parse_twitter_window_YTD_bullshit(archive.accountjs)
    account = parsed_account[0]["account"]
    username = account["username"]
    accountid = account["accountId"]
    displayname = account["accountDisplayName"]

    parsed_profile = parse_twitter_window_YTD_bullshit(archive.profilejs)
    profile = parsed_profile[0]["profile"]
    pfp_url_filename = profile["avatarMediaUrl"].split("/")[-1]
    pfp_filename = f"{accountid}-{pfp_url_filename}"
    pfp_path = os.path.join(archive.profilemedia, pfp_filename)
    with open(pfp_path, "br") as pfpfp:
        pfp_bytes = pfpfp.read()

    parsed_tweets = parse_twitter_window_YTD_bullshit(archive.tweetjs)

    infltweets: typing.List[InflatedTweet] = []
    for outertweet in parsed_tweets:
        tweet = outertweet["tweet"]
        tweetid = tweet["id_str"]
        logprefix = f"Tweet {tweetid} in archive {archive.path}"
        if twitter_archive_tweet_is_low_fidelity_retweet(tweet):
            if api:
                logger.info(
                    f"{logprefix} is a low fidelity retweet, will try to download the original from Twitter..."
                )
                twitterapi.tweet2data_continue_on_error(
                    site,
                    api,
                    tweetid,
                    None,
                    force=api_force_download,
                    max_rlevel=max_recurse,
                )
            else:
                logger.info(
                    f"{logprefix} is low fidelity retweet and api argument was not passed, skipping..."
                )
        else:
            logger.info(f"{logprefix} is a regular tweet, saving to disk...")
            infltweet = inflated_tweet_from_twitter_archive(
                tweet,
                pfp_bytes,
                username,
                displayname,
                archive,
            )
            filename = os.path.join(site.data_twarchive, f"{tweetid}.json")
            infltweet.jdump(filepath=filename)
            infltweets.append(infltweet)



"""Code used exclusively for testing"""

import os
import shutil
import tempfile

from twarchive import hugo


class TemporaryHugoSite:
    """A context manager that creates a temporary Hugo site

    Creates the required (empty) directories for a hugo.HugoSite to exist
    """

    def __init__(self):
        self.site = hugo.HugoSite(tempfile.mkdtemp())
        os.mkdir(self.site.content)
        os.mkdir(self.site.content_twarchive)
        os.mkdir(self.site.data)
        os.mkdir(self.site.data_twarchive)

    def __enter__(self):
        return self.site

    def __exit__(self, exc_type, exc_value, exc_tb):
        shutil.rmtree(self.site.base)



import setuptools


long_description = """
# twarchive

A companion package for the twarchive Hugo theme.
This package makes it easy to download tweets in the format that the Hugo theme expects.

For more information, see
<https://github.com/mrled/twarchive>
"""


version_ns = {}
with open("twarchive/version.py") as vfp:
    exec(vfp.read(), version_ns)


setuptools.setup(
    name="twarchive",
    version=version_ns["__version__"],
    author="Micah R Ledbetter",
    author_email="me@micahrl.com",
    description="Download tweets for use with the twarchive Hugo theme",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/mrled/twarchive/",
    packages=["twarchive"],
    python_requires=">=3.9",
    include_package_data=True,
    install_requires=["tweepy"],
    setup_requires=[
        "black",
        "mypy",
        "pytest",
    ],
    entry_points={
        "console_scripts": ["twarchive=twarchive.cli:main"],
    },
)



#!/usr/bin/env python3

import logging

from twarchive import version


logging.basicConfig(
    level=logging.INFO, format="[%(asctime)s] [%(name)s] [%(levelname)s] %(message)s"
)
logger = logging.getLogger(__name__)


__version__ = version.__version__
